@@ -1,33 +1,4 @@
 /* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* enchant
- * Copyright (C) 2003 Dom Lachowicz
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- *
- * In addition, as a special exception, Dom Lachowicz
- * gives permission to link the code of this program with
- * non-LGPL Spelling Provider libraries (eg: a MSFT Office
- * spell checker backend) and distribute linked combinations including
- * the two.  You must obey the GNU Lesser General Public License in all
- * respects for all of the code used other than said providers.  If you modify
- * this file, you may extend this exception to your version of the
- * file, but you are not obligated to do so.  If you do not wish to
- * do so, delete this exception statement from your version.
- */
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h> 
@@ -115,8 +86,9 @@ ISpellChecker::try_autodetect_charset(const char * const inEncoding)
 /***************************************************************************/
 /***************************************************************************/
 
-ISpellChecker::ISpellChecker()
-	: deftflag(-1),
+ISpellChecker::ISpellChecker(EnchantBroker * broker)
+: m_broker(broker),
+	deftflag(-1),
      prefstringchar(-1),
      m_bSuccessfulInit(false),
      m_BC(NULL),
@@ -207,8 +179,8 @@ ISpellChecker::checkWord(const char * const utf8Word, size_t length)
 	
 	if (!strtoichar(iWord, szWord, sizeof(iWord), 0))
 		{
-			if (good(iWord, 0, 0, 1, 0) == 1 ||
-			    compoundgood(iWord, 1) == 1)
+			if (good(iWord, 0, 0, 1, 0) >= 1 ||
+			    compoundgood(iWord, 1))
 				{
 					retVal = true;
 				}
@@ -301,7 +273,7 @@ ISpellChecker::suggestWord(const char * const utf8Word, size_t length,
 }
 
 static GSList *
-ispell_checker_get_dictionary_dirs (void)
+ispell_checker_get_dictionary_dirs (EnchantBroker * broker)
 {
 	GSList *dirs = NULL;
 
@@ -320,6 +292,18 @@ ispell_checker_get_dictionary_dirs (void)
 		g_slist_free (config_dirs);
 	}
 
+#if 0
+	{
+		const gchar* const * system_data_dirs = g_get_system_data_dirs ();
+		const gchar* const * iter;
+
+		for (iter = system_data_dirs; *iter; iter++)
+			{
+				dirs = g_slist_append (dirs, g_build_filename (*iter, "ispell", "dicts", NULL));
+			}
+	}
+#endif
+
 	/* until I work out how to link the modules against enchant in MacOSX - fjf
 	 */
 #ifndef XP_TARGET_COCOA
@@ -344,17 +328,31 @@ ispell_checker_get_dictionary_dirs (void)
 	dirs = g_slist_append (dirs, g_strdup (ENCHANT_ISPELL_DICT_DIR));
 #endif
 
+	{
+		GSList *config_dirs, *iter;
+
+		config_dirs = enchant_get_dirs_from_param (broker, "enchant.ispell.dictionary.path");
+		
+		for (iter = config_dirs; iter; iter = iter->next)
+			{
+				dirs = g_slist_append (dirs, g_strdup ((const gchar *)iter->data));
+			}
+
+		g_slist_foreach (config_dirs, (GFunc)g_free, NULL);
+		g_slist_free (config_dirs);
+	}
+
 	return dirs;
 }
 
 static void
-s_buildHashNames (std::vector<std::string> & names, const char * dict)
+s_buildHashNames (std::vector<std::string> & names, EnchantBroker * broker, const char * dict)
 {
 	names.clear ();
 
 	GSList *dirs, *iter;
 
-	dirs = ispell_checker_get_dictionary_dirs();
+	dirs = ispell_checker_get_dictionary_dirs(broker);
 	for (iter = dirs; iter; iter = iter->next)
 		{
 			char *tmp;
@@ -373,7 +371,7 @@ ISpellChecker::loadDictionary (const char * szdict)
 {
 	std::vector<std::string> dict_names;
 
-	s_buildHashNames (dict_names, szdict);
+	s_buildHashNames (dict_names, m_broker, szdict);
 
 	for (size_t i = 0; i < dict_names.size(); i++)
 		{
@@ -438,11 +436,11 @@ ISpellChecker::setDictionaryEncoding( const char * hashname, const char * encodi
 			
 			if (prefstringchar < 0)
 				{
-					std::string teststring;
+					char teststring[64];
 					for(int n1 = 1; n1 <= 15; n1++)
 						{
-							teststring = "latin" + n1;
-							prefstringchar = findfiletype(teststring.c_str(), 1,				      
+							sprintf(teststring, "latin%d", n1);
+							prefstringchar = findfiletype(teststring, 1,				      
 										      deftflag < 0 ? &deftflag : static_cast<int *>(NULL));
 							if (prefstringchar >= 0) 
 								break;
@@ -550,7 +548,7 @@ ispell_provider_request_dict (EnchantProvider * me, const char *const tag)
 	EnchantDict *dict;
 	ISpellChecker * checker;
 	
-	checker = new ISpellChecker ();
+	checker = new ISpellChecker (me->owner);
 	
 	if (!checker)
 		{
@@ -583,11 +581,11 @@ ispell_provider_dispose_dict (EnchantProvider * me, EnchantDict * dict)
 }
 
 static int
-_ispell_provider_dictionary_exists (const char *const szFile)
+_ispell_provider_dictionary_exists (EnchantBroker * broker, const char *const szFile)
 {
 	std::vector <std::string> names;
 
-	s_buildHashNames (names, szFile);
+	s_buildHashNames (names, broker, szFile);
 	for (size_t i = 0; i < names.size(); i++) {
 		if (g_file_test (names[i].c_str(), G_FILE_TEST_EXISTS))
 			return 1;
@@ -596,6 +594,11 @@ _ispell_provider_dictionary_exists (const char *const szFile)
 	return 0;
 }
 
+extern "C" {
+
+ENCHANT_MODULE_EXPORT (EnchantProvider *) 
+	     init_enchant_provider (void);
+
 static char **
 ispell_provider_list_dictionaries (EnchantProvider * me,
 				   size_t * out_n_dicts)
@@ -607,7 +610,7 @@ ispell_provider_list_dictionaries (EnchantProvider * me,
 
 	nb = 0;
 	for (i = 0; i < size_ispell_map; i++)
-		if (_ispell_provider_dictionary_exists (ispell_map[i].dict))
+		if (_ispell_provider_dictionary_exists (me->owner, ispell_map[i].dict))
 			out_dicts[nb++] = g_strdup (ispell_map[i].lang);
 
 	*out_n_dicts = nb;
@@ -632,7 +635,7 @@ ispell_provider_dictionary_exists (struct str_enchant_provider * me,
 		{
 			const IspellMap * mapping = (const IspellMap *)(&(ispell_map[i]));
 			if (!strcmp (tag, mapping->lang) || !strcmp (shortened_dict.c_str(), mapping->lang)) 
-				return _ispell_provider_dictionary_exists(mapping->dict);
+				return _ispell_provider_dictionary_exists(me->owner, mapping->dict);
 		}
 	
 	return 0;
@@ -662,11 +665,6 @@ ispell_provider_describe (EnchantProvider * me)
 	return "Ispell Provider";
 }
 
-extern "C" {
-
-ENCHANT_MODULE_EXPORT (EnchantProvider *) 
-	     init_enchant_provider (void);
-
 EnchantProvider *
 init_enchant_provider (void)
 {
